import uuid
from datetime import timedelta

import pytest
from django.utils import timezone

from django_retention_policy import (
    task_delete_expired_db_records,
)
from tests.example_project.example_app import models

pytestmark = pytest.mark.django_db


class TestTaskDeleteExpiredDbRecords:
    def test(self, freezer):
        data_retention_num_seconds = 86400
        now = timezone.now()

        records_within_retention_period_timestamps = (
            now,
            now - timedelta(seconds=data_retention_num_seconds),
            now - timedelta(seconds=data_retention_num_seconds - 1),
            now - timedelta(seconds=data_retention_num_seconds - 2),
            now - timedelta(seconds=data_retention_num_seconds - 100000),
        )
        records_outside_retention_period_timestamps = (
            now - timedelta(seconds=data_retention_num_seconds + 1),
            now - timedelta(seconds=data_retention_num_seconds + 2),
            now - timedelta(seconds=data_retention_num_seconds + 3),
            now - timedelta(seconds=data_retention_num_seconds + 100000),
        )

        records_within_retention_period = []
        for timestamp in records_within_retention_period_timestamps:
            freezer.move_to(timestamp)
            record = models.ExampleModel.objects.create(
                user_id=str(uuid.uuid4()),
                timestamp=timestamp,
            )
            records_within_retention_period.append(record)

        records_outside_retention_period = []
        for timestamp in records_outside_retention_period_timestamps:
            freezer.move_to(timestamp)
            record = models.ExampleModel.objects.create(
                user_id=str(uuid.uuid4()),
                timestamp=timestamp,
            )
            records_outside_retention_period.append(record)

        result = task_delete_expired_db_records(
            app_name='example_app',
            model_name='ExampleModel',
            time_based_column_name='timestamp',
            data_retention_num_seconds=data_retention_num_seconds,
            now=now,
        )

        assert result == (
            len(records_outside_retention_period),
            {
                'example_app.ExampleModel': len(
                    records_outside_retention_period
                )
            },
        )
        record_pks = models.ExampleModel.objects.all().values_list(
            'pk', flat=True
        )
        assert set(record_pks) == set(
            r.pk for r in records_within_retention_period
        )



from django.utils import timezone
from django.db import models


class ExampleModel(models.Model):
    user_id = models.CharField(max_length=36)
    created_on = models.DateTimeField(default=timezone.now)
    modified_on = models.DateTimeField(auto_now=True)
    timestamp = models.DateTimeField(primary_key=True, auto_now_add=True)

    class Meta:
        db_table = 'example_model'
        unique_together = (
            (
                'user_id',
                'timestamp',
                'type',
            ),
        )
        indexes = [
            models.Index(
                name='%(class)s_idx',
                fields=[
                    'user_id',
                    'type',
                    'timestamp',
                ],
            )
        ]



# Generated by Django 4.0.6 on 2022-08-10 10:45

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='ExampleModel',
            fields=[
                ('user_id', models.CharField(max_length=36)),
                (
                    'created_on',
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                ('modified_on', models.DateTimeField(auto_now=True)),
                (
                    'timestamp',
                    models.DateTimeField(
                        auto_now_add=True, primary_key=True, serialize=False
                    ),
                ),
            ],
            options={
                'db_table': 'example_model',
            },
        ),
        migrations.AddIndex(
            model_name='examplemodel',
            index=models.Index(
                fields=['user_id', 'timestamp'], name='examplemodel_idx'
            ),
        ),
        migrations.AlterUniqueTogether(
            name='examplemodel',
            unique_together={('user_id', 'timestamp')},
        ),
    ]



import logging
import time
from datetime import timedelta

from django.apps import apps
from django.utils import timezone
from celery import shared_task

logger = logging.getLogger(__name__)


@shared_task
def task_delete_expired_db_records(
    app_name,
    model_name,
    time_based_column_name,
    data_retention_num_seconds,
    now=None,
):
    logger.info(
        'Checking for %s %s DB records to delete which are '
        'beyond retention period...',
        app_name,
        model_name,
    )
    start_time = time.time()
    model_class = apps.get_model(app_label=app_name, model_name=model_name)
    now = now or timezone.now()
    min_age_records_to_delete = now - timedelta(
        seconds=data_retention_num_seconds
    )
    logger.info(
        'Executing delete query on %s %s records older than %s...',
        app_name,
        model_name,
        min_age_records_to_delete,
    )
    deleted_records = model_class.objects.filter(
        **{f'{time_based_column_name}__lt': min_age_records_to_delete}
    ).delete()
    took = time.time() - start_time
    logger.info(
        'Done deleting %s %s DB records to delete which are '
        'beyond retention period. Took %s secs. Result: %s',
        app_name,
        model_name,
        took,
        deleted_records,
    )
    return deleted_records
